@@ -17,11 +17,13 @@ static d_open_t  nvidia_ctl_open;
 static void nvidia_ctl_dtor(void *arg);
 static d_ioctl_t nvidia_ctl_ioctl;
 static d_poll_t  nvidia_ctl_poll;
+static d_mmap_single_t nvidia_ctl_mmap_single;
 
 static struct cdevsw nvidia_ctl_cdevsw = {
     .d_open =      nvidia_ctl_open,
     .d_ioctl =     nvidia_ctl_ioctl,
     .d_poll =      nvidia_ctl_poll,
+    .d_mmap_single = nvidia_ctl_mmap_single,
     .d_name =      "nvidiactl",
     .d_version =   D_VERSION,
 };
@@ -35,7 +37,7 @@ static int nvidia_ctl_open(
     struct cdev *dev,
     int oflags,
     int devtype,
-    d_thread_t *td
+    struct thread *td
 )
 {
     int status;
@@ -88,7 +90,7 @@ static int nvidia_ctl_ioctl(
     u_long cmd,
     caddr_t data,
     int fflag,
-    d_thread_t *td
+    struct thread *td
 )
 {
     int status;
@@ -112,30 +114,23 @@ static int nvidia_ctl_ioctl(
 static int nvidia_ctl_poll(
     struct cdev *dev,
     int events,
-    d_thread_t *td
+    struct thread *td
 )
 {
     struct nvidia_filep *filep;
     int status, mask = 0;
-    struct nvidia_event *et;
 
     status = devfs_get_cdevpriv((void **)&filep);
     if (status != 0)
         return 0;
 
     mtx_lock(&filep->event_mtx);
-    et = STAILQ_FIRST(&filep->event_queue);
 
-    if (et == NULL) {
+    if (STAILQ_EMPTY(&filep->event_queue) && !filep->event_pending)
         selrecord(td, &filep->event_rsel);
-        et = STAILQ_FIRST(&filep->event_queue);
-    }
-    else
+    else {
         mask = (events & (POLLIN | POLLPRI | POLLRDNORM));
-
-    if ((et != NULL) && ((et->event.flags & NV_EVENT_FLAGS_DATA_VALID) == 0)) {
-        STAILQ_REMOVE(&filep->event_queue, et, nvidia_event, queue);
-        free(et, M_NVIDIA);
+        filep->event_pending = FALSE;
     }
 
     mtx_unlock(&filep->event_mtx);
@@ -143,6 +138,30 @@ static int nvidia_ctl_poll(
     return mask;
 }
 
+static int nvidia_ctl_mmap_single(
+    struct cdev *dev,
+    vm_ooffset_t *offset,
+    vm_size_t size,
+    vm_object_t *object,
+    int nprot
+)
+{
+    int status;
+    struct nvidia_filep *filep;
+    nv_state_t *nv;
+
+    status = devfs_get_cdevpriv((void **)&filep);
+    if (status != 0)
+        return status;
+    nv = filep->nv;
+
+    nv_lock_api(nv);
+    status = nvidia_mmap_ctl_single(nv, filep, offset, size, object);
+    nv_unlock_api(nv);
+
+    return status;
+}
+
 int nvidia_ctl_attach(void)
 {
     if (nvidia_count == 0) {
