@@ -33,7 +33,7 @@ static int nvidia_dev_open(
     struct cdev *dev,
     int oflags,
     int devtype,
-    d_thread_t *td
+    struct thread *td
 )
 {
     int status;
@@ -87,7 +87,7 @@ static int nvidia_dev_ioctl(
     u_long cmd,
     caddr_t data,
     int fflag,
-    d_thread_t *td
+    struct thread *td
 )
 {
     int status;
@@ -112,30 +112,23 @@ static int nvidia_dev_ioctl(
 static int nvidia_dev_poll(
     struct cdev *dev,
     int events,
-    d_thread_t *td
+    struct thread *td
 )
 {
     struct nvidia_filep *filep;
     int status, mask = 0;
-    struct nvidia_event *et;
 
     status = devfs_get_cdevpriv((void **)&filep);
     if (status != 0)
         return 0;
 
     mtx_lock(&filep->event_mtx);
-    et = STAILQ_FIRST(&filep->event_queue);
 
-    if (et == NULL) {
+    if (STAILQ_EMPTY(&filep->event_queue) && !filep->event_pending)
         selrecord(td, &filep->event_rsel);
-        et = STAILQ_FIRST(&filep->event_queue);
-    }
-    else
+    else {
         mask = (events & (POLLIN | POLLPRI | POLLRDNORM));
-
-    if ((et != NULL) && ((et->event.flags & NV_EVENT_FLAGS_DATA_VALID) == 0)) {
-        STAILQ_REMOVE(&filep->event_queue, et, nvidia_event, queue);
-        free(et, M_NVIDIA);
+        filep->event_pending = FALSE;
     }
 
     mtx_unlock(&filep->event_mtx);
@@ -161,7 +154,7 @@ static int nvidia_dev_mmap_single(
     nv = filep->nv;
 
     nv_lock_api(nv);
-    status = nvidia_mmap_dev_single(nv, offset, size, object);
+    status = nvidia_mmap_dev_single(nv, filep, offset, size, object);
     nv_unlock_api(nv);
 
     return status;
